[PATCH v2 0/2] fetch: speed up mirror-fetches with many refs
- From
Patrick Steinhardt <ps@pks.im>
- Date
- Feb 2, 2022, 12:51 UTC
- Message-ID
- <cover.1643806143.git.ps@pks.im>
- In-Reply-To
- <cover.1643364888.git.ps@pks.im>
Hi,
this is the second version of my patch series which aims to speed up mirror-fetches in repos with huge amounts of refs. The only change compared to v1 is a fixed up commit message: Taylor has pointed out to me that commit dates retrieved from the commit-graph are not in fact the corrected commit dates, which are stored separately.
Patrick
Patrick Steinhardt (2): fetch-pack: use commit-graph when computing cutoff fetch: skip computing output width when not printing anything
builtin/fetch.c | 8 ++++++-- fetch-pack.c | 28 ++++++++++++++++------------ 2 files changed, 22 insertions(+), 14 deletions(-)
Range-diff against v1:
1: 31cf8f87a1 ! 1: 6fac914f0f fetch-pack: use commit-graph when computing cutoff
@@ Commit message
the cutoff date. This can be sped up by trying to look up commits via
the commit-graph first, which is a lot more efficient.
- One thing to keep in mind though is that the commit-graph corrects
- committer dates:
-
- * A commit with at least one parent has corrected committer date
- equal to the maximum of its commiter date and one more than the
- largest corrected committer date among its parents.
-
- As a result, it may be that the commit date we load via the commit graph
- is more recent than it would have been when loaded via the ODB, and as a
- result we may also choose a more recent cutoff point. But as the code
- documents, this is only a heuristic and it is okay if we determine a
- wrong cutoff date. The worst that can happen is that we report more
- commits as HAVEs to the server when using corrected dates.
-
- Loading commits via the commit-graph is typically much faster than
- loading commits via the object database. Benchmarks in a repository with
- about 2,1 million refs and an up-to-date commit-graph show a 20% speedup
- when mirror-fetching:
+ Benchmarks in a repository with about 2,1 million refs and an up-to-date
+ commit-graph show a 20% speedup when mirror-fetching:
Benchmark 1: git fetch --atomic +refs/*:refs/* (v2.35.0)
Time (mean ± σ): 75.264 s ± 1.115 s [User: 68.199 s, System: 10.094 s]
2: 5a3fd3232f = 2: 4b9bbcf795 fetch: skip computing output width when not printing anything-- 2.35.1